//////////////////////////////////////////////////////////////////////
/*
   35 pixels
   Copyright 2021 - Eric Sérandour
   Site Web : https://www.serandour.com
*/
   const String VERSION = "2021.05.02"; // 21 h 08
/*   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
//////////////////////////////////////////////////////////////////////
/*
   Compile sans problème avec Arduino 1.8.15.
*/
//////////////////////////////////////////////////////////////////////
/*
  L'électronique :

// Cablage du MAX7219
// 5V   (fil marron) => alimentation externe
// GND  (fil rouge)
// DIN  (fil orange) => broche 10 de l'Arduino
// LOAD (fil jaune)  => broche 11 de l'Arduino
// CLK  (fil vert)   => broche 12 de l'Arduino

*/
//////////////////////////////////////////////////////////////////////

const byte MAX7219_DIN = 10;
const byte MAX7219_LOAD = 11;
const byte MAX7219_CLK = 12;

#include "table.h"

// Mémoire tampon :
const int numX = 3;  // Virtuellement 3 zones de 8 pixels en horizontal
const int numY = 3;  // Virtuellement 3 zones de 8 pixels en vertical
const int sizeBuffer = (8 * numX) * (8 * numY);
boolean bufferMem[sizeBuffer];  // Cache
/* 
 *  La mémoire tampon est virtuellement découpée en 9 zones numérotées de 0 à 8 (numCache) :
 *  [0][1][2]
 *  [3][4][5]
 *  [6][7][8]
 *  
 *  Chaque zone ci-dessus représente une matrice virtuelle 8x8 : 
 *  [  0   7] [  8  15] [ 16  23]
 *  [       ] [       ] [       ]
 *  [168 175] [176 183] [184 191]
 *  
 *  [192 199] [200 207] [208 215]
 *  [       ] [       ] [       ]
 *  [360 367] [368 375] [376 383]
 *  
 *  [384 391] [392 399] [400 407]
 *  [       ] [       ] [       ]
 *  [552 559] [560 567] [568 575]
 *  
 *  La matrice globale correspond à bufferMem[0...575].
 *  
 *  La matrice de leds (FENETRE) est centrée sur la zone 4.
 *  
 */
const byte FENETRE = 4;  // Fenêtre de visualisation sur la mémoire tampon (correspond à la matrice de leds)

const int MAXI_STEPS = sizeBuffer;  // Nombre de pas maximum pour les procédures shiftChar

boolean passage = false;
int compteur = 0;



//////////////////////////////////////////////////////////////////////

void writeMAX7219(byte address, byte data) // Voir Timing Diagram dans le datasheet   // OK
{
  digitalWrite(MAX7219_LOAD, LOW);
  for (int d=15; d>=8; d--) {
    digitalWrite(MAX7219_CLK, LOW);
    digitalWrite(MAX7219_DIN, address & B10000000);
    address = address << 1;
    digitalWrite(MAX7219_CLK, HIGH);
  }  
  for (int d=7; d>=0; d--) {
    digitalWrite(MAX7219_CLK, LOW);
    digitalWrite(MAX7219_DIN, data & B10000000);
    data = data << 1;
    digitalWrite(MAX7219_CLK, HIGH);
  }   
  digitalWrite(MAX7219_LOAD, HIGH);
}

//////////////////////////////////////////////////////////////////////

void initMAX7219() // Voir Register Address Map dans le datasheet                   // OK
{
  writeMAX7219(0x09, 0x00); // Decode Mode
  writeMAX7219(0x0a, 0x0f); // Intensity de 0x00 à 0x0f
  writeMAX7219(0x0b, 0x07); // Scan Limit
  writeMAX7219(0x0c, 0x01); // Shutdown
  writeMAX7219(0x0f, 0x00); // Display Test
}

//////////////////////////////////////////////////////////////////////

void clearMatrix()                                                                 // OK
// Efface la matrice
{
  for (byte address=0x01; address<=0x08; address++) {
    writeMAX7219(address, B00000000);
  }
}

//////////////////////////////////////////////////////////////////////

int indice(int x, int y)                                                            // OK
// Convertit des coordonnées (x;y) en indice pour le tampon
{
  return 8 * numX * y + x;
}

//////////////////////////////////////////////////////////////////////

void affecterCoordonneesIndice (int indice, int& x, int& y)     // OK
// Convertit un indice du tampon en coordonnées (x;y)
// Passage par référence : La modification de la variable affecte également la valeur de la variable en dehors de la fonction
{
  x = indice % (8 * numX);  // Reste de la division euclidienne
  y = indice / (8 * numX);  // Quotient de la division euclidienne
}

//////////////////////////////////////////////////////////////////////

 



//////////////////////////////////////////////////////////////////////

void setup()
{
  Serial.begin(9600);
  pinMode(MAX7219_DIN, OUTPUT);
  pinMode(MAX7219_LOAD, OUTPUT);
  pinMode(MAX7219_CLK, OUTPUT);
  delay(50);
  initMAX7219();
  clearMatrix();
  randomSeed(analogRead(0));
} 

//////////////////////////////////////////////////////////////////////

// Loop
void loop()
{
  char carL;
  char carC;
  if (!passage) {
    
    String lettre = "EEEEEEEEEEEEEAAAAAAAAIIIIIIISSSSSSSNNNNNNNRRRRRRRTTTTTTOOOOOOLLLLLUUUUUDDDDCCCCMMMPPPGGBBVVHHFFQYXJKWZ";
    String chiffre = "9876543210";
    String texte = "POEME";
    int sizeLettre = lettre.length();
    int sizeChiffre = chiffre.length();
    int sizeTexte = texte.length();

    carL = lettre[random(sizeLettre)];
    int delayTime = random(300);
    if (random(2)<1) {
      carC = chiffre[random(sizeChiffre)];
    }
      
    switch (random(4)) {
      case 0: shiftCharLeft(carL, 5, 16, delayTime); break;
      case 1: shiftCharRight(carC, 3, 16, 150); break;
      case 2: shiftCharDown(carL, 1, 8, delayTime); break;
      case 3: shiftCharUp(carC, 7, 8, 150); break;  
    }
    if (random(10)<1) {
      scrollText(texte, FENETRE, 150);
    }
    passage = false;
  } 
}

//////////////////////////////////////////////////////////////////////
